#! /usr/lib/veetest/veetest -r
(revision "A.00.01")
(date "Wed 18/Sep/1991")
(component 0 "Context"
 (name "Feedback5 Example")
 (interface
 )
 (implementation
  (trigMode deg)
  (locked no)
  (component 0 "ExitThread"
   (interface
    (sequence in)
   )
   (implementation
   )
   (views
    (icon
     (origin 795 678)
     (extent 110 25)
    )
    (detail
    )
    (terminals on)
    (active icon)
   )
  )
  (component 1 "IfThenElse"
   (interface
    (sequence in)
    (sequence out)
    (input 1
     (type data)
     (name "A")
     (optional yes)
    )
    (output 1
     (type data)
     (name "Then")
     (lock name constraints)
    )
    (output 2
     (type data)
     (name "Else")
     (lock name constraints)
    )
   )
   (implementation
    (component isLocked no)
    (component expr "A==4")
   )
   (views
    (icon
     (origin 711 628)
     (extent 78 25)
    )
    (detail
     (origin 613 585)
     (extent 64 70)
    )
    (terminals on)
    (active icon)
   )
  )
  (component 2 "Counter"
   (interface
    (sequence in)
    (sequence out)
    (input 1
     (type data)
     (name "Data")
    )
    (input 2
     (type control)
     (name "Clear")
     (lock name constraints)
     (optional yes)
    )
    (output 1
     (type data)
     (name "Count")
     (tag "Count")
     (lock name constraints)
    )
   )
   (implementation
    (component clearAtPrerun 1)
    (component clearAtActivate 1)
   )
   (views
    (icon
     (origin 611 628)
     (extent 78 25)
    )
    (detail
     (origin 605 623)
     (extent 150 34)
    )
    (active icon)
   )
  )
  (component 3 "Gate"
   (interface
    (sequence in)
    (sequence out)
    (input 1
     (type data)
     (name "A")
     (tag "InData")
    )
    (output 1
     (type data)
     (name "X")
     (tag "OutData")
     (lock name constraints)
    )
   )
   (implementation
   )
   (views
    (icon
     (origin 307 418)
     (extent 46 25)
    )
    (detail
    )
    (terminals on)
    (active icon)
   )
  )
  (component 4 "Formula"
   (name "A+1")
   (interface
    (sequence in)
    (sequence out)
    (input 1
     (type data)
     (name "A")
     (optional yes)
    )
    (output 1
     (type data)
     (name "Result")
     (lock name constraints)
    )
   )
   (implementation
    (component isLocked no)
    (component expr 1 "A+1")
   )
   (views
    (icon
     (origin 427 468)
     (extent 46 25)
    )
    (detail
     (origin 606 630)
     (extent 79 60)
    )
    (terminals on)
    (active icon)
   )
  )
  (component 5 "ForCount"
   (interface
    (sequence in)
    (sequence out)
    (output 1
     (type data)
     (lock name constraints)
    )
   )
   (implementation
    (component count 4)
   )
   (views
    (icon
     (origin 215 418)
     (extent 110 64)
     (iconImage "loop.icon")
    )
    (detail
     (origin 195 563)
     (extent 170 34)
    )
    (active detail)
   )
  )
  (component 6 "Constant"
   (name "Integer")
   (interface
    (sequence in)
    (sequence out)
    (output 1
     (type data)
     (lock name constraints)
    )
   )
   (implementation
    (component value Int32
     (name "Integer")
     (numDims 1)
     (size 5)
     (data
      [ 1 20 5 18 2 ]
     )
    )
    (component autoTrigger off)
    (component initializeAtPrerun off)
    (component initializeAtActivate off)
    (component initValue Int32
     (datum 0)
    )
   )
   (views
    (icon
    )
    (detail
     (origin 25 331)
     (extent 150 118)
     (showFormat int)
    )
    (active detail)
   )
  )
  (component 7 "IfThenElse"
   (name "A[B+1]<A[B]")
   (interface
    (sequence in)
    (sequence out)
    (input 1
     (type data)
     (name "A")
     (optional yes)
    )
    (input 2
     (type data)
     (name "B")
     (optional yes)
    )
    (output 1
     (type data)
     (name "Then")
     (lock name constraints)
    )
    (output 2
     (type data)
     (name "Else")
     (lock name constraints)
    )
   )
   (implementation
    (component isLocked no)
    (component expr "A[B+1]<A[B]")
   )
   (views
    (icon
     (origin 454 358)
     (extent 112 25)
    )
    (detail
     (origin 324 198)
     (extent 163 104)
    )
    (terminals on)
    (active icon)
   )
  )
  (component 8 "Junction"
   (name "JCT")
   (interface
    (input 1
     (type data)
     (name "A")
     (lock name constraints)
    )
    (input 2
     (type data)
     (name "B")
     (lock name constraints)
    )
    (input 3
     (type data)
     (name "C")
     (optional yes)
    )
    (output 1
     (type data)
     (name "Data")
     (lock name constraints)
    )
   )
   (implementation
   )
   (views
    (icon
     (origin 227 343)
     (extent 46 34)
    )
    (detail
    )
    (terminals on)
    (active icon)
   )
  )
  (component 9 "RepeatUntilBreak"
   (interface
    (sequence in)
    (sequence out)
    (output 1
     (type data)
     (name "Continuous")
     (lock name constraints)
    )
   )
   (implementation
   )
   (views
    (icon
     (origin 45 478)
     (extent 110 64)
     (iconImage "loop.icon")
    )
    (detail
    )
    (terminals on)
    (active icon)
   )
  )
  (component 10 "Context"
   (name "Swap(x,y)")
   (description 9 4
    This model inside the User Object is explained in
    the /examples/concepts directory, It is used to
    swap any two elements of an array. The array can
    be any of the Vee data types. The inputs are the
    array, two of whose elements are to be swapped. 
    The other two inputs are the indices of the two 
    elements. The indices should be attached in the 
    correct order as indicated by the terminal names.
    
   )
   (interface
    (sequence in)
    (sequence out)
    (input 1
     (type data)
     (name "ArrayIn")
     (optional yes)
    )
    (input 2
     (type data)
     (name "Xindex")
     (optional yes)
    )
    (input 3
     (type data)
     (name "Yindex")
     (optional yes)
    )
    (output 1
     (type data)
     (name "SwappedArray")
     (lock constraints)
     (optional yes)
    )
   )
   (implementation
    (trigMode deg)
    (locked no)
    (component 0 "ToArray"
     (interface
      (sequence in)
      (sequence out)
      (input 1
       (type data)
       (name "Array")
       (tag "Array")
       (lock constraints)
      )
      (input 2
       (type data)
       (name "Datum")
       (tag "Datum")
       (requires
        (shape "Scalar")
       )
       (lock constraints)
      )
      (input 3
       (type data)
       (name "Index 1")
       (tag "Index 1")
       (requires
        (datatype Int32)
        (shape "Scalar")
       )
       (lock constraints)
      )
      (input 4
       (type trigger)
       (name "XEQ")
       (lock name constraints)
      )
      (output 1
       (type data)
       (name "Array")
       (tag "Array")
       (lock constraints)
      )
     )
     (implementation
      (component numDims 1)
     )
     (views
      (icon
       (origin 655 368)
       (extent 110 64)
       (iconImage "build.icon")
      )
      (detail
      )
      (terminals on)
      (active icon)
     )
    )
    (component 1 "Formula"
     (interface
      (sequence in)
      (sequence out)
      (input 1
       (type data)
       (name "A")
       (optional yes)
      )
      (input 2
       (type data)
       (name "B")
       (optional yes)
      )
      (output 1
       (type data)
       (name "Result")
       (lock name constraints)
      )
     )
     (implementation
      (component isLocked no)
      (component expr 1 "A[B]")
     )
     (views
      (icon
       (origin 341 358)
       (extent 78 64)
       (iconImage "formula.icon")
      )
      (detail
       (origin 308 166)
       (extent 74 108)
      )
      (terminals on)
      (active icon)
     )
    )
    (component 2 "Formula"
     (interface
      (sequence in)
      (sequence out)
      (input 1
       (type data)
       (name "A")
       (optional yes)
      )
      (input 2
       (type data)
       (name "B")
       (optional yes)
      )
      (output 1
       (type data)
       (name "Result")
       (lock name constraints)
      )
     )
     (implementation
      (component isLocked no)
      (component expr 1 "A[B]")
     )
     (views
      (icon
       (origin 341 178)
       (extent 78 64)
       (iconImage "formula.icon")
      )
      (detail
       (origin 308 166)
       (extent 74 108)
      )
      (terminals on)
      (active icon)
     )
    )
    (component 3 "ToArray"
     (interface
      (sequence in)
      (sequence out)
      (input 1
       (type data)
       (name "Array")
       (tag "Array")
       (lock constraints)
      )
      (input 2
       (type data)
       (name "Datum")
       (tag "Datum")
       (requires
        (shape "Scalar")
       )
       (lock constraints)
      )
      (input 3
       (type data)
       (name "Index 1")
       (tag "Index 1")
       (requires
        (datatype Int32)
        (shape "Scalar")
       )
       (lock constraints)
      )
      (input 4
       (type trigger)
       (name "XEQ")
       (lock name constraints)
      )
      (output 1
       (type data)
       (name "Array")
       (tag "Array")
       (lock constraints)
      )
     )
     (implementation
      (component numDims 1)
     )
     (views
      (icon
       (origin 495 148)
       (extent 110 64)
       (iconImage "build.icon")
      )
      (detail
      )
      (terminals on)
      (active icon)
     )
    )
    (configuration
     (connect D3:1 D0:1)
     (connect D1:1 D0:2)
     (connect I2:1 D0:3)
     (connect D3:0 D0:4)
     (connect I1:1 D1:1)
     (connect I3:1 D1:2)
     (connect I1:1 D2:1)
     (connect I2:1 D2:2)
     (connect I1:1 D3:1)
     (connect D2:1 D3:2)
     (connect I3:1 D3:3)
     (connect D2:0 D3:4)
     (connect D0:1 O1:1)
    )
   )
   (views
    (icon
     (origin 620 428)
     (extent 100 45)
    )
    (detail
     (origin 175 117)
     (extent 641 387)
     (configuration
      (connect D3:1 D0:1
       (Point 607 180)
       (Point 620 180)
       (Point 620 380)
       (Point 652 380)
      )
      (connect D1:1 D0:2
       (Point 421 390)
       (Point 652 390)
      )
      (connect I2:1 D0:3
       (Point 177 320)
       (Point 200 320)
       (Point 200 440)
       (Point 440 440)
       (Point 440 400)
       (Point 652 400)
      )
      (connect D3:0 D0:4
       (Point 550 214)
       (Point 550 410)
       (Point 652 410)
      )
      (connect I1:1 D1:1
       (Point 177 190)
       (Point 310 190)
       (Point 310 370)
       (Point 338 370)
      )
      (connect I3:1 D1:2
       (Point 177 450)
       (Point 190 450)
       (Point 190 410)
       (Point 338 410)
      )
      (connect I1:1 D2:1
       (Point 177 190)
       (Point 338 190)
      )
      (connect I2:1 D2:2
       (Point 177 320)
       (Point 190 320)
       (Point 190 230)
       (Point 338 230)
      )
      (connect I1:1 D3:1
       (Point 177 190)
       (Point 310 190)
       (Point 310 160)
       (Point 492 160)
      )
      (connect D2:1 D3:2
       (Point 421 210)
       (Point 440 210)
       (Point 440 170)
       (Point 492 170)
      )
      (connect I3:1 D3:3
       (Point 177 450)
       (Point 190 450)
       (Point 190 410)
       (Point 320 410)
       (Point 320 340)
       (Point 460 340)
       (Point 460 180)
       (Point 492 180)
      )
      (connect D2:0 D3:4
       (Point 380 244)
       (Point 380 260)
       (Point 450 260)
       (Point 450 190)
       (Point 492 190)
      )
      (connect D0:1 O1:1
       (Point 767 400)
       (Point 780 400)
       (Point 780 320)
       (Point 813 320)
      )
     )
    )
    (terminals on)
    (active icon)
   )
  )
  (component 11 "TextDisplay"
   (name "AlphaNumeric")
   (interface
    (sequence in)
    (sequence out)
    (input 1
     (type data)
     (name "Data")
    )
   )
   (implementation
    (component clearAtPrerun 1)
    (component clearAtActivate 1)
   )
   (views
    (icon
    )
    (detail
     (origin 794 351)
     (extent 192 198)
    )
    (active detail)
   )
  )
  (component 12 "SampleHold"
   (interface
    (sequence in)
    (sequence out)
    (input 1
     (type data)
     (name "InData")
     (tag "InData")
     (lock constraints)
    )
    (input 2
     (type trigger)
     (name "XEQ")
     (lock name constraints)
    )
    (output 1
     (type data)
     (name "OutData")
     (tag "OutData")
     (lock constraints)
    )
   )
   (implementation
   )
   (views
    (icon
     (origin 597 338)
     (extent 127 25)
    )
    (detail
    )
    (terminals on)
    (active icon)
   )
  )
  (component 13 "Context"
   (name "Press For")
   (interface
    (sequence in)
    (sequence out)
   )
   (implementation
    (trigMode deg)
    (locked no)
    (component 0 "IfThenElse"
     (name "!= 0")
     (interface
      (sequence in)
      (sequence out)
      (input 1
       (type data)
       (name "A")
       (optional yes)
      )
      (output 1
       (type data)
       (name "Then")
       (lock name constraints)
      )
      (output 2
       (type data)
       (name "Else")
       (lock name constraints)
      )
     )
     (implementation
      (component isLocked no)
      (component expr "a!=0")
     )
     (views
      (icon
       (origin 385 340)
       (extent 46 25)
      )
      (detail
       (origin 209 308)
       (extent 73 64)
      )
      (terminals on)
      (active icon)
     )
    )
    (component 1 "Toggle"
     (name "Info About")
     (interface
      (sequence in)
      (sequence out)
      (input 1
       (type control)
       (name "Reset")
       (lock name constraints)
       (optional yes)
      )
      (output 1
       (type data)
       (lock name constraints)
      )
     )
     (implementation
      (component value Int32
       (name "Info About")
       (values range 0 1)
       (datum 0)
      )
      (component autoTrigger on)
      (component initializeAtPrerun off)
      (component initializeAtActivate off)
      (component initValue Int32
       (datum 0)
      )
     )
     (views
      (icon
       (origin 221 333)
       (extent 114 39)
       (format button)
      )
      (detail
       (origin 320 203)
       (extent 42 54)
      )
      (terminals on)
      (active icon)
     )
    )
    (component 2 "Context"
     (name "Info About This Example")
     (interface
      (sequence in)
      (sequence out)
     )
     (implementation
      (trigMode deg)
      (locked no)
      (component 0 "Note"
       (name "NotePad")
       (description 2 8
        This notepad is displayed as a Show on Execute
        panel to present the information to the reader.
       )
       (interface
       )
       (implementation
        (component text 45 9
         PURPOSE:     To show a more complex feedback loop involving actual constraints of objects
                      within the feedback loop.
         
         USER INPUT REQUIRED: None
         
         EXPLANATION: This example implements a simple sort algorithm called a bubble sort.  Simply
                      put, the bubble sort is used to sort an array of elements by stepping 
                      through the array and comparing the element at position i with the element
                      at position i+1.  If the element at position i+1 is less than the element
                      at position i (out of order), then the two elements are swapped. The object
                      used to swap the elements is in the library directory.  The algorithm 
                      continually steps through the array making comparisons and swapping until
                      the elements are in order. 
         
                      This algorithm is implemented in HP Vee using a feedback loop.  The data
                      within the feedback loop is the array of elements to be sorted.  The array
                      loops through the feedback loop having each two adjacent elements compared
                      and swapped.  Notice that there are two iterators, an Until Break and a
                      For Count.  The For Count is used to step through each element of the array,
                      the Until Break is used to drive the For Count so that enough sweeps through
                      the array are done to achieve the correct order.  The object making the 
                      comparisons is an If/Then retitled to "A[B+1] < A[B]".  If this comparison
                      is true, i.e. the two adjacent elements are out of order, then the swap
                      object is allowed to occur after executing the Sample and Hold to pass the
                      most recent array to the swap User Object.  
                      
                      The "else" terminal is attached to a gate object so that if the comparison
                      fails, i.e. the elements are in order, then the unchanged array is feed back
                      to the beginning of the feedback loop.  This very important. Note that if the
                      elements are in order the swap object is not allowed to execute.  In that case
                      the unchanged array must be sent back to the Junction object.  There are three
                      asynchronous data input pins on the Junction object, one for the initial un-
                      sorted array, one for the array after its elements have been swapped, and one
                      for the array if the elements are in order.
         
                      The "else" terminal of the If/Then is attached to a Counter object.  If
                      the Counter is activated four times by the "else" pin then that means that
                      there were no adjacent elements out of order.  The second If/Then attached to
                      the Counter contains "A==4".  If this is true the array is sorted and the 
                      thread is stopped. See Chapter 3 in the Users Guide for more information
                      concerning feedback loops.
         
                      Turn on Execution and Data Flow under the Edit menu and watch this model
                      execute.
         
        )
       )
       (views
        (icon
         (origin 250 435)
         (extent 78 25)
        )
        (detail
         (origin 36 162)
         (extent 848 437)
         (editing enabled)
        )
        (active icon)
       )
      )
      (component 1 "Confirm"
       (name "DONE")
       (description 7 8
        This tiny "thread" begins running when the 
        enclosing UserObject is executed.  When the user
        acknowledges the OK/DONE button, there is nothing
        left to execute, so the thread ends.  Since there
        are not other threads running, the User Object
        is completed and will exit, removing the Show on
        Execute pop-up display.
       )
       (interface
        (sequence in)
        (sequence out)
        (output 1
         (type data)
         (name "Go")
         (lock name constraints)
        )
       )
       (implementation
       )
       (views
        (icon
         (origin 261 502)
         (extent 58 37)
        )
        (detail
         (origin 156 253)
         (extent 79 34)
        )
        (terminals on)
        (active icon)
       )
      )
      (configuration
      )
      (ShowOnExecPanel
       (origin 55 78)
       (extent 878 573)
       (panel
        (origin 73 163)
        (extent 874 535)
        (widget 1 icon
         (title off)
         (origin 472 648)
         (extent 58 37)
        )
        (widget 0 detail
         (title off)
         (origin 83 177)
         (extent 853 462)
         (editing disabled)
        )
       )
      )
     )
     (views
      (icon
       (origin 402 418)
       (extent 217 25)
      )
      (detail
       (origin 107 390)
       (extent 866 520)
       (configuration
       )
      )
      (active icon)
     )
    )
    (configuration
     (connect D1:1 D0:1)
     (connect D0:0 D1:1)
     (connect D0:1 D2:0)
    )
   )
   (views
    (icon
     (origin 175 368)
     (extent 110 25)
    )
    (panel
     (origin 15 119)
     (extent 291 82)
     (widget 1 icon
      (title off)
      (origin 93 143)
      (extent 114 39)
      (format button)
     )
    )
    (detail
     (origin 54 115)
     (extent 912 591)
     (configuration
      (connect D1:1 D0:1
       (Point 337 352)
       (Point 382 352)
      )
      (connect D0:0 D1:1
       (Point 408 367)
       (Point 408 390)
       (Point 200 390)
       (Point 200 352)
       (Point 218 352)
      )
      (connect D0:1 D2:0
       (Point 433 342)
       (Point 510 342)
       (Point 510 415)
      )
     )
    )
    (active panel)
    (active panel)
   )
  )
  (component 14 "Note"
   (name "Please Note !!!")
   (interface
   )
   (implementation
    (component text 5 5
         This example is to demostrate a 
       feedback technique.  This is NOT an
        optimized sort routine. To see an
        optimized sorting example, please
     reference: examples/concepts/qsort.ex
    )
   )
   (views
    (icon
     (extent 145 0)
     (iconImage "notepad.icon")
    )
    (detail
     (origin 325 118)
     (extent 351 105)
     (editing enabled)
    )
    (active detail)
   )
  )
  (component 15 "Note"
   (name "Warranty Disclaimer")
   (interface
   )
   (implementation
    (component text 5 5
     This example is provided as an
     illustration "as is", and 
     Hewlett-PackardCompany makes no
     warranty of any kind with regard
      to this example.
    )
   )
   (views
    (icon
     (extent 181 0)
    )
    (detail
     (origin 686 118)
     (extent 309 105)
     (editing disabled)
    )
    (active detail)
   )
  )
  (component 16 "Start"
   (interface
    (sequence out)
   )
   (implementation
   )
   (views
    (icon
     (origin 67 232)
     (extent 67 37)
    )
    (detail
    )
    (terminals on)
    (active icon)
   )
  )
  (configuration
   (connect D1:1 D0:0)
   (connect D2:1 D1:1)
   (connect D7:2 D2:1)
   (connect D5:0 D2:2)
   (connect D7:2 D3:0)
   (connect D8:1 D3:1)
   (connect D5:1 D4:1)
   (connect D9:1 D5:0)
   (connect D16:0 D6:0)
   (connect D8:1 D7:1)
   (connect D5:1 D7:2)
   (connect D10:1 D8:1)
   (connect D6:1 D8:2)
   (connect D3:1 D8:3)
   (connect D6:0 D9:0)
   (connect D12:1 D10:1)
   (connect D5:1 D10:2)
   (connect D4:1 D10:3)
   (connect D10:1 D11:1)
   (connect D8:1 D12:1)
   (connect D7:1 D12:2)
  )
 )
 (views
  (detail
   (origin 0 67)
   (extent 1004 664)
   (configuration
    (connect D1:1 D0:0
     (Point 791 630)
     (Point 850 630)
     (Point 850 675)
    )
    (connect D2:1 D1:1
     (Point 691 640)
     (Point 708 640)
    )
    (connect D7:2 D2:1
     (Point 568 380)
     (Point 580 380)
     (Point 580 630)
     (Point 608 630)
    )
    (connect D5:0 D2:2
     (Point 280 599)
     (Point 280 650)
     (Point 608 650)
    )
    (connect D7:2 D3:0
     (Point 568 380)
     (Point 580 380)
     (Point 580 400)
     (Point 330 400)
     (Point 330 415)
    )
    (connect D8:1 D3:1
     (Point 275 360)
     (Point 290 360)
     (Point 290 430)
     (Point 304 430)
    )
    (connect D5:1 D4:1
     (Point 367 580)
     (Point 390 580)
     (Point 390 480)
     (Point 424 480)
    )
    (connect D9:1 D5:0
     (Point 157 510)
     (Point 280 510)
     (Point 280 530)
    )
    (connect D16:0 D6:0
     (Point 100 271)
     (Point 100 298)
    )
    (connect D8:1 D7:1
     (Point 275 360)
     (Point 451 360)
    )
    (connect D5:1 D7:2
     (Point 367 580)
     (Point 390 580)
     (Point 390 480)
     (Point 410 480)
     (Point 410 380)
     (Point 451 380)
    )
    (connect D10:1 D8:1
     (Point 722 450)
     (Point 760 450)
     (Point 760 320)
     (Point 210 320)
     (Point 210 350)
     (Point 224 350)
    )
    (connect D6:1 D8:2
     (Point 177 390)
     (Point 200 390)
     (Point 200 360)
     (Point 224 360)
    )
    (connect D3:1 D8:3
     (Point 355 430)
     (Point 370 430)
     (Point 370 460)
     (Point 210 460)
     (Point 210 370)
     (Point 224 370)
    )
    (connect D6:0 D9:0
     (Point 100 451)
     (Point 100 475)
    )
    (connect D12:1 D10:1
     (Point 726 350)
     (Point 740 350)
     (Point 740 380)
     (Point 600 380)
     (Point 600 440)
     (Point 617 440)
    )
    (connect D5:1 D10:2
     (Point 367 580)
     (Point 390 580)
     (Point 390 480)
     (Point 410 480)
     (Point 410 450)
     (Point 617 450)
    )
    (connect D4:1 D10:3
     (Point 475 480)
     (Point 490 480)
     (Point 490 460)
     (Point 617 460)
    )
    (connect D10:1 D11:1
     (Point 722 450)
     (Point 791 450)
    )
    (connect D8:1 D12:1
     (Point 275 360)
     (Point 430 360)
     (Point 430 340)
     (Point 594 340)
    )
    (connect D7:1 D12:2
     (Point 568 360)
     (Point 594 360)
    )
   )
  )
  (active detail)
  (numberFormats
   (realFormat standard)
   (realSigDigits 4)
   (realRadixSpec 4)
   (integerBase decimal)
  )
  (waveformSettings
   (timeSpan 0.02)
   (numPoints 256)
  )
 )
)
